grammar TRA;

@lexer::header {
package de.tum.tra.parser;
}
@header {
package de.tum.tra.parser;
import java.util.List;
import java.util.ArrayList;
import de.tum.tra.terms.*;
}

@parser::members {
  @Override
  public void reportError(RecognitionException e) {
      String[] tokenNames = getTokenNames();
      String hdr = getErrorHeader(e);
      String msg = getErrorMessage(e, tokenNames);
      throw new InternalParseError(hdr+" "+msg,e);
  }
}

@lexer::members {
  @Override
  public void reportError(RecognitionException e) {
      String[] tokenNames = getTokenNames();
      String hdr = getErrorHeader(e);
      String msg = getErrorMessage(e, tokenNames);
      throw new InternalParseError(hdr+" "+msg,e);
  }
}

ID  :   ('a'..'z'|'A'..'Z')('a'..'z'|'A'..'Z'|'_'|'\''|'0'..'9')* ;
INT :   '0'..'9'+ ;
WS  :   (' '|'\t'|'\n')+ {skip();} ;

equation returns [App e]:
  l=expr '==' r=expr { $e = new App("==", $l.t, $r.t); }
  ;

exprStart returns [Node t]: e=expr EOF { $t = $e.t; };

expr returns [Node t]
    : e=eqExpr{ $t = $e.t; }
    ;
    
eqExpr returns [Node t]
   : { Node rhs=null; } l=orExpr ('=?=' r=addExpr { rhs = $r.t; })? {
   if (rhs == null)
      return $l.t;
   else
      return new App("=?=", $l.t, rhs);
   }
   ;
   
orExpr returns [Node t]
  : e=andExpr { $t = $e.t; } ('\\/' e=andExpr { $t = new App("\\/", $t, $e.t); })*
  ;

andExpr returns [Node t]
  : e=addExpr { $t = $e.t; } ('/\\' e=addExpr { $t = new App("/\\", $t, $e.t); })*
  ;

addExpr returns [Node t]
    : e=multExpr { $t = $e.t; }
        (   '+' e=multExpr { $t = new App("+", $t, $e.t); }
        |   '-' e=multExpr { $t = new App("-", $t, $e.t);  }
        )*
    ;

multExpr returns [Node t]
    :   e=consExpr {$t = $e.t;} ('*' e=consExpr {$t = new App("*", $t, $e.t);})*        
    ; 

consExpr returns [Node t]
    : e=atom {$t = $e.t; } ('.' e = consExpr { $t = new App(".", $t, $e.t); })?
    ;

atom returns [Node t]
    :   INT {$t = new App($INT.text, App.EMPTY_ARGS);}
    |  { List<Node> args = null; }  
        ID ('(' { args = new ArrayList<Node>(5); }
          (e = expr { args.add($e.t); } (',' e=expr { args.add($e.t); })*)? ')')? 
      { if (args == null) {
          if (Character.isUpperCase($ID.text.charAt(0)))
            $t = new Var($ID.text);
          else
            $t = new App($ID.text, App.EMPTY_ARGS);
        } else
          $t = new App($ID.text, args.toArray(new Node[args.size()]));
      }
    |   '(' e=expr ')' {$t = $e.t;}
    ;
